from argparse import ArgumentParser
import os

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import h5py

FIGSIZE=(8, 5)

def read_samples(filename):
    with h5py.File(filename, "r") as ff:
        data = ff["samples"][:]
        events = [event.decode() for event in ff["events"]]
    data_frame = pd.DataFrame()
    for name in data.dtype.names:
        data_frame[name] = data[name]
    return data_frame, events

def set_matplotlibrc_parameters():
    from matplotlib import rcParams
    rcParams["font.family"] = "serif"
    rcParams["font.serif"] = "Computer Modern Roman"
    rcParams["font.size"] = 20
    rcParams["text.usetex"] = True
    rcParams["grid.alpha"] = 0


def plot_predicted_histogram(
    data, key, save=True, filename="predicted_histogram.png", **hist_kwargs
):
    kwargs = dict(
        bins=100,
        density=True,
        histtype="step",
    )
    kwargs.update(hist_kwargs)
    plt.hist(data.flatten(), **kwargs)
    plt.xlabel("$m_1$ [$M_{\\odot}$]")
    plt.ylabel("p($m_1$) [$M_{\\odot}^{{-1}}$]")
    plt.tight_layout()
    if save:
        plt.savefig(filename)
        plt.close()
    else:
        return fig


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("--sample-file", help="Sample file in common format")
    args = parser.parse_args()
    
    data, events = read_samples(args.sample_file)

    n_events = len(events)
    parameters = [
        "_".join(key.split("_")[1:]) for key in data
        if events[0] in key and "prior" not in key
    ]

    observed = {
        key: np.array([
            data[f"{name}_{key}"] for name in events
            if f"{name}_{key}" in data
        ])
        for key in parameters
        if any([f"{name}_{key}" in data for name in events])
    }
    
    synthetic = {
        key: np.array([
            data[f"synthetic_{key}_{ii}"] for ii in range(len(events))
            if f"synthetic_{key}_{ii}" in data
        ])
        for key in parameters
        if any([f"synthetic_{key}_{ii}" in data for ii in range(len(events))])
    }

    base_name = os.path.splitext(os.path.split(args.sample_file)[1])[0]
    
    set_matplotlibrc_parameters()

    parameter = "mass_1"
    print(base_name)
    plot_predicted_histogram(
        synthetic[parameter],
        parameter,
        filename=f"{base_name}_{parameter}_predicted.png"
    )
